from CyXTraX.simulation import CylindricalProjection, load_mesh, SAVEMODES, utility
from CyXTraX.io import load_atlas
from CyXTraX.mapping import map_geometry_2_projection, map_source_2_cylinder
from CyXTraX.util.visualisation import voxel_cone_maps

from pathlib import Path
from jax import numpy as jnp
from jax.scipy.spatial.transform import Rotation

from cyxtrax_object import set_geometry_at_index, circular_trajectory, add_world_orign, zero_quat, x_90_quat, z_90_quat, add_offset



# Just get the temp folder to store the atlas
FOLDER = Path(__file__)
TEMP_FOLDER = FOLDER.parent.parent / 'temp'
MAP_FOLDER = Path(r'C:\data\XrayTransformLow')


def main():
    files = list(MAP_FOLDER.glob('*.h5'))
    if not files:
        raise FileNotFoundError('No .h5 files in temp folder. Run Example 02!')

    maps, points, mesh_object_list = load_atlas(files[3])
    cyl_proj = CylindricalProjection()
    
    load_mesh(cyl_proj, mesh_object_list, False)
    print(f'Cone Beam Mode set: {cyl_proj.cone_mode}')


    number_of_projections = 30

    fod_mm = jnp.array([1000,])
    fdd_mm = jnp.array([2000,])
    alpha_rad = jnp.linspace(0, jnp.pi, number_of_projections).reshape((-1, 1))

    source, orientation_source, detector, orientation_detector = circular_trajectory(fod_mm, fdd_mm, alpha_rad)
    source, orientation_source, detector, orientation_detector = add_world_orign(
        source, orientation_source, detector, orientation_detector, jnp.array([1, 2., -1.]), jnp.array([0.8, 0.1, -0.1, 0.9]))
    detector, orientation_detector = add_offset(detector, orientation_detector, jnp.array([1, 20., -1.]), zero_quat)

    temp_projection = Path('temp.tif')
    

    
    for i in range(number_of_projections):
        set_geometry_at_index(source, orientation_source, detector, orientation_detector, i, temp_projection)

        projection, geometry = utility.load_projection(temp_projection)
        mapped_values, uv_px = map_geometry_2_projection(
                source[i], 
                detector[i],
                orientation_detector[i],
                points,
                projection, 
                cyl_proj.pixel_pitch_mm)
            
        values_calc, angles_calc = map_source_2_cylinder(
            source[i], 
            maps, 
            points, 
            cyl_proj.detector_radius_mm, 
            cyl_proj.x_px,
            cyl_proj.x_resolution_mm)
        
        voxel_cone_maps(maps, projection, uv_px, angles_calc, TEMP_FOLDER / f'{i:03}_presentation.png')
    

if __name__ == '__main__':
    main()